/*
 **********************************************************************************************************************************************
 *  Copyright 2005/2006, Pascal Pfiffner.                                                                                                     *
 *                                                                                                                                            *
 *  This Widget is Freeware and will always be. However, if you`d like to use some of its code, please tell me at "help.you.i.can@gmail.com"  *
 **********************************************************************************************************************************************
 */


// Globals
var theBall;
var leftBar;
var rightBar;
var scoreLeft = 0;
var scoreRight = 0;
var gameOver = true;
var gameLevel = 4;
var thisVersion = "0.3";

// Settings
var mode = "clock";						// "clock" or "game"
var clocktype = getClockType();			// 12- or 24-hours clock
var bouncemode = getBounceMode();		// relative or absolute
var showglass = getShowGlass();			// show or hide the glass?
var ballControl = {
				ballSize:6,
				bounceDist:18,			// distance of the bar-surface (where the ball hits) to the border of the playfield
				fieldTotalWidth:150,	// width of the field
				fieldWidth:108,			// width from bar to bar (inner distance bar to bar _MINUS_ the ball-width, used for the x-value of the ball)
				fieldTotalHeight:110,	// full height
				fieldHeight:104,		// full height minus the ball size (used to give the ball an y-value)
				
				speed:1000,			// ms to perform one length of the playfield (till the player hits)
				startAngle:30,		// positive angle means down, negative angle means up
				maxAngle:65,		// maximum allowed angle
				yRounded:0			// saves how many decimals were rounded while calculating the next Y-value
			};
var gameLevels = new Array();
gameLevels[0] = 1400;
gameLevels[1] = 1300;
gameLevels[2] = 1200;
gameLevels[3] = 1100;
gameLevels[4] = 1000;
gameLevels[5] = 900;
gameLevels[6] = 800;
gameLevels[7] = 700;
gameLevels[8] = 600;
gameLevels[9] = 500;
gameLevels[10] = 400;
gameLevels[11] = 300;
gameLevels[12] = 200;

// Globals for Apple
var flipShown = false;
var animation = { timer:null, duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, secondElement:null };


function showVersion() {
	getObj("infospan").innerHTML = thisVersion;
}


// Dashboard Setup
function setup() {
	showGlass(showglass);
	showTime();
	
	// widget-events
	if(window.widget) {
		widget.onshow = onShow;
		widget.onhide = onHide;
		widget.onremove = doCleanup;
	}
	
	// images
	for(var i = 0; i < 10; i++) {
		var img = new Image();
		img.src = "./Images/" + i + ".gif";
	}
	
	// start
	theBall = new Ball();
	leftBar = new Player('left');
	rightBar = new Player('right');
	switchPrefTab('widget');
	createGenericButton(getObj("done"), "done", savePrefs);
	theBall.run = true;
	theBall.startTimed();
}



function toggleMode(event) {
	mode = ("clock" == mode) ? "game" : "clock";
	var toggle = getObj("modeToggle");
	
	if(mode == "game") {
		stop();
		startGame();
		togglePause(1);
		document.addEventListener("keydown", keyDown, true);
		document.addEventListener("keyup", keyUp, true);
		event.stopPropagation();
		event.preventDefault();
		toggle.src = "Images/clock.gif";
		toggle.alt = "clock";
	}
	else {
		stop();
		gameOver = true;
		togglePause(2);
		theBall.speed = 1000;
		rightBar.setCompetitionSpeed();
		leftBar.setCompetitionSpeed();
		getObj("losesign").style.display = 'none';
		getObj("winsign").style.display = 'none';
		document.removeEventListener("keydown", keyDown, true);
		document.removeEventListener("keyup", keyUp, true);
		event.stopPropagation();
		event.preventDefault();
		toggle.src = "Images/play.gif";
		toggle.alt = "play!";
	}
	
	return;
}


var paused = false;
function togglePause(force) {		// if force is 1, force a pause, if it is anything but false or 1, force a start
	var pausesign = getObj("pausesign");
	if(force) {
		paused = (1 == force) ? false : true;
	}
	
	paused = !paused;
	
	if(paused) {
		stop();
		if('clock' != mode)
			pausesign.style.display = 'block';
	}
	else {
		play();
		pausesign.style.display = 'none';
	}
}

function startGame() {
	if('game' != mode)
		return;
	
	gameOver = false;
	getObj("losesign").style.display = 'none';
	getObj("winsign").style.display = 'none';
	togglePause(2);
	updateScore(1);
}


// if we pass an argument, the score gets reset. Else we count on.
function updateScore(init) {
	if('game' != mode)
		return;
	
	if(!init && !gameOver) {
		if(1 == theBall.direction)
			scoreLeft++;
		else
			scoreRight++;
	}
	else {
		scoreLeft = 0;
		scoreRight = 0;
	}
	
	displayNumber(-1, scoreLeft);
	displayNumber(1, scoreRight);
	
	if((scoreLeft < 10) && (scoreRight < 10)) {
		theBall.init();
		setTimeout("theBall.start()", 2500);
	}
	else {
		var sign = null;
		
		// you win
		if(scoreRight > scoreLeft) {
			gameLevel++;
			sign = getObj("winsign");
		}
		
		// you lose
		else {
			gameLevel--;
			gameLevel = (gameLevel < 0) ? 0 : gameLevel;
			sign = getObj("losesign");
		}
		
		getLastNode(sign, "SPAN").innerHTML = gameLevel;
		sign.style.display = 'block';
		gameOver = true;
		
		// change gamespeed
		theBall.speed = gameLevels[gameLevel];
		rightBar.setCompetitionSpeed();
		leftBar.setCompetitionSpeed();
	}
}

function showTime(hour, minute) {
	if('clock' != mode)
		return;
	
	if(!hour)
		hour = (new Date()).getHours();
	if(!minute)
		minute = (new Date()).getMinutes();
	
	if(clocktype == 12)
		hour = hour % 12;
	
	scoreLeft = hour;
	scoreRight = minute;
	
	displayNumber(-1, scoreLeft);
	displayNumber(1, scoreRight);
}

function displayNumber(where, number) {			// 1 means right, -1 means left
	if(! where)
		where = 1;
	if(isNaN(number))
		return;
	
	var obj = (where == 1) ? getObj("scoreRight") : getObj("scoreLeft");
	var strlen = number.toString().length;
	
	// remove old images
	while(obj.lastChild)
		obj.removeChild(obj.lastChild);
	
	// set new images
	for(var i = 0; i < strlen; i++) {
		var img = document.createElement("img");
		img.setAttribute("class", "number");
		img.src = "./Images/" + number.toString().substring(i, i + 1) + ".gif";
		obj.appendChild(img);
	}
}

var keyPressed = false;
function keyDown(event) {
	if('game' != mode)
		return;
	
	if(gameOver) {
		startGame();
		return;
	}
	
	if(paused) {
		togglePause();
		return;
	}
	
	var which = (event.which == 40) ? "downR" : ((event.which == 38) ? "upR" : "pause");
	
	keyPressed = which;
	switch(which) {
		case "upR":
		case "downR":
		case "upL":
		case "downL":
			rightBar.starttime = (new Date()).getTime();
			rightBar.moveManual();
			break;
		
		default:
			togglePause();
			break;
	}
}

function keyUp(event) {
	keyPressed = false;
}










/*
 ****************************
 *      Pref-functions      *
 ****************************
 */


function getClockType() {
	var type = 24;
	if(window.widget) {
		type = widget.preferenceForKey('clocktype');
		type = (24 == type || 12 == type) ? type : 24;
	}
	
	return type;
}

function getBounceMode() {
	var mode = 'relative';
	if(window.widget) {
		mode = widget.preferenceForKey('bouncemode');
		mode = (mode == 'absolute') ? mode : 'relative';
	}
	
	return mode;
}

function getShowGlass() {
	var show = false;
	if(window.widget) {
		show = widget.preferenceForKey('showglass');
		show = show ? true : false;
	}
	
	return show;
}


function showPrefs() {
	var front = getObj("front");
	var back = getObj("back");
	
	adjustPrefs();		// adjusts the prefs
	showVersion();		// sets version-number
	
	if(window.widget)
		widget.prepareForTransition("ToBack");
	
	front.style.display = "none";
	back.style.display = "block";
	
	togglePause(1);
	
	if(window.widget)
		setTimeout("widget.performTransition();", 0);
}


function adjustPrefs() {
	var prefform = getObj('prefform');
	
	prefform.elements['clocktype'][(clocktype == 12) ? 0 : 1].checked = true;
	prefform.elements['bouncemode'][(bouncemode == 'absolute') ? 1 : 0].checked = true;
	prefform.elements['showglass'].checked = showglass;
}

function switchPrefTab(tab) {
	var found = false;
	var thedivs = new Array("widget", "game", "about");
	
	for(var i = 0; i < thedivs.length; i++) {
		if(thedivs[i] == tab) {
			getObj("tab_" + thedivs[i]).setAttribute("class", "tabitem selected");
			getObj("prefsection_" + thedivs[i]).style.display = "block";
			found = true;
		}
		else {
			getObj("tab_" + thedivs[i]).setAttribute("class", "tabitem");
			getObj("prefsection_" + thedivs[i]).style.display = "none";
		}
	}
	
	if(!found) {
		getObj("prefsection_widget").style.display = "block";
	}
}

function savePrefs() {
	var prefform = getObj('prefform');
	
	// get clock type
	var clocktypes = prefform.elements['clocktype'];
	clocktype = clocktypes[0].checked ? 12 : 24;
	
	// get bounce mode
	var bouncemodes = prefform.elements['bouncemode'];
	bouncemode = bouncemodes[0].checked ? 'relative' : 'absolute';
	
	// get if glass should be shown
	showglass = prefform.elements['showglass'].checked;
	showGlass(showglass);
	
	// save all
	if(window.widget) {
		widget.setPreferenceForKey(clocktype, 'clocktype');
		widget.setPreferenceForKey(bouncemode, 'bouncemode');
		widget.setPreferenceForKey(showglass, 'showglass');
	}
	
	hidePrefs();
}


function hidePrefs() {
	var front = getObj("front");
	var back = getObj("back");
	
	if(window.widget)
		widget.prepareForTransition("ToFront");
	
	//getObj("fliprollie").style.display = "none";	
	front.style.display = "block";
	back.style.display = "none";
	
	setTimeout("play()", 600);
	
	if(window.widget)
		setTimeout("widget.performTransition();", 0);
}


function showGlass(sure) {
	var glass = getObj('glass');
	
	if(sure) {
		glass.style.display = 'block';
	}
	else {
		glass.style.display = 'none';
	}
}


function selectVal(clicked) {
	var elem = clicked;
	while(null != elem.previousSibling) {
		if('INPUT' == elem.previousSibling.nodeName) {
			elem.previousSibling.checked = ! elem.previousSibling.checked;
			return;
		}
		elem = elem.previousSibling;
	}
}


function checkLatestVersion() {
	requestPage("http://code.hillrippers.ch/widget_pongClock/latest");
}

// sends a request for a page
function requestPage(url) {
	req = createXMLHttpRequest();
	if(!req)
		return false;
	
	req.onreadystatechange = handleStateChange;
	req.open("GET", url, true);
	req.send(null);
	return true;
}

// handles the readystatechanges
function handleStateChange() {
	if(!req)
		return false;
	
	switch(req.readyState) {
		case 0:		// UNINITIALIZED
		case 1:		// LOADING
		case 2:		// LOADED
		case 3:		// INTERACTIVE
			break;
		case 4:		// COMPLETED
			receivePage(req.status, req.responseText);
			break;
		default:;
	}
}

// receives a requested page
function receivePage(status, text) {
	var latest = text;
	latest = latest ? latest.replace(/[^-\d\.]/, "") : 0.1;
	var myVersion = thisVersion.replace(/[^-\d\.]/, "");
	var info = getObj('latestinfo');
	
	info.innerHTML = (latest == myVersion) ? ("You are up to date") : ((myVersion > latest) ? ("Beta-Tester, right?") : ("Latest version is " + latest));
}

// returns a new XMLHttpRequest-Object. Also serves the MSIE, maybe the widget runs in this messy browser, too?
function createXMLHttpRequest() {
	var req = null;
	try {
		req = new ActiveXObject("MSXML2.XMLHTTP");
	}
	catch(err_MSXML2) {
		try {
			req = new ActiveXObject("Microsoft.XMLHTTP");
		}
		catch(err_Microsoft) {
			if(typeof(XMLHttpRequest) != "undefined") {
				req = new XMLHttpRequest;
			}
		}
	}
	
	return req;
}



function onShow() {
	play();
}

function onHide() {
	stop();
}

function doCleanup() {
	return;
}

function getObj(id) {
	return document.getElementById(id);
}

function getLastNode(obj, name) {
	if(!obj)
		return null;
	
	var arr = obj.childNodes;
	if(!name)
		return arr[arr.length - 1];
	
	for(var i = arr.length; i > 0; i--) {
		if(arr[i-1].nodeName == name)
			return arr[i-1];
	}
}











/*
 *********************************
 *      Technical functions      *
 *********************************
 */

// calculates delta-y from delta-x and the given angle.
function getDeltaY(deltaX, advanced_functions) {
	var angle = theBall.angle;
	if(!deltaX || deltaX == 0)
		return 0;
	
	deltaX = Math.abs(deltaX);
	
	var deltaY = 0;
	var deltaYfull = eval(deltaX * Math.tan(theBall.angle * Math.PI / 180));
	var direction = 1;			// 1 is down (add deltaY to old Y-value)
	if(deltaYfull < 0) {
		direction = -1;
		deltaYfull = Math.abs(deltaYfull);
	}
	
	if(advanced_functions)
		deltaYfull = Math.abs(deltaYfull + ballControl.yRounded);
	
	while(deltaYfull > 1) {
		deltaYfull -= 1;
		deltaY += 1;
	}
	
	if(advanced_functions)
		ballControl.yRounded = deltaYfull;
	
	return deltaY * direction;
}


function play() {
	if(!theBall)
		return;
	
	theBall.run = true;
	if('clock' == mode) {
		showTime();
		theBall.start();
	}
	else if(!paused && !gameOver) {
		theBall.start();
	}
}

function stop() {
	if(!theBall)
		return;
	
	theBall.run = false;
	theBall.moving = false;
}




/*
 ***********************************************
 *      Moving the Bars; the Player-Class      *
 ***********************************************
 */
var playerBar = {
			barHeight:24,			// height of the bars
			
			speed:400,				// after these milliseconds from the starttime, the player is at its destination (in clock-mode)
			competitionSpeed:0.65	// that much time of the total ball-time needs the human player to move the bar from top to bottom
		};


function Player(side) {
	if( ! (this instanceof Player))		// ensure we successfully created the Player
		return new Player();
	
	this.side = ('left' == side) ? 'left' : 'right';
	this.height = playerBar.barHeight;
	this.y = 30;
	this.bottomY = this.y + this.height;
	this.x = ('left' == this.side) ? (ballControl.bounceDist - ballControl.ballSize) : (ballControl.fieldTotalWidth - ballControl.bounceDist);
	this.rightX = 1* this.x + ballControl.ballSize;
	this.surfaceX = ('left' == this.side) ? this.rightX : (this.x - theBall.size);

	this.start = this.y;
	this.aim = 70;
	this.anglePercentageChange = 0.15;		// on the outmost hit-point, the angle of the ball changes this much compared to his old angle (in relative mode)
	
	this.starttime = 0;
	this.endtime = 0;
	this.timeout = null;
	this.speed = playerBar.speed;
	this.competitionSpeed = 650;
	this.setCompetitionSpeed();
	
	this.div = document.createElement('div');
	this.div.setAttribute("class", "bar");
	this.div.setAttribute("style", "top:" + this.y + "px; left:" + this.x + "px;");
	getObj('playField').appendChild(this.div);
}
Player.prototype.getX = getX;
Player.prototype.getY = getY;
Player.prototype.setY = playerSetY;
Player.prototype.startAutoMove = playerStartAutoMove;
Player.prototype.moveTimed = playerMoveTimed;
Player.prototype.moveCompetition = playerMoveCompetition;
Player.prototype.moveManual = playerMoveManual;
Player.prototype.setCompetitionSpeed = playerSetCompetitionSpeed;


function getX() {
	return this.x;
}

function getY() {
	return this.y;
}

// sets a new y-value for the bar. Takes care of overshooting the borders of the playfield.
function playerSetY(y) {
	var newY = (y < 0) ? 0 : ((y > Math.abs(ballControl.fieldTotalHeight - this.height)) ? Math.abs(ballControl.fieldTotalHeight - this.height) : y);
	this.y = newY;
	this.bottomY = newY + this.height;
	this.div.style.top = newY + 'px';
}

function playerStartAutoMove() {
	var now = new Date();

	if('game' == mode) {
		if(('left' == this.side) && (-1 == theBall.direction)) {
			this.starttime = now.getTime();
			this.moveCompetition();
		}
		return;
	}
	
	var new_hour = (now.getMinutes() == 59);
	var too_late = (("clock" == mode) && (now.getSeconds() >= 57));
	
	// lets the ball hit at the center of the bar
	var centerOffset = Math.round((this.height - theBall.size) / 2);
	
	// lets the ball hit at a random point on the bar (half the bar-height will be randomizable; due to imperfectness of the Y-prediction, the rest is a security-margin)
	var randOffset = Math.round((Math.random() * (this.height / 2)) - (this.height / 4));
	
	// set start and aim
	this.start = this.y;
	this.aim = theBall.projectedY - centerOffset + randOffset;
	if(('left' == this.side) && !new_hour && too_late)
		this.aim = (this.aim < 40) ? (this.aim + 40) : (this.aim - 40);			// left player misses every minute but 59th
	else if(('right' == this.side) && new_hour && too_late)
		this.aim = (this.aim < 40) ? (this.aim + 40) : (this.aim - 40);			// on new hour, right player misses
	
	this.starttime = (new Date()).getTime();
	this.endtime = this.starttime + this.speed;
	this.moveTimed();
}


function playerMoveTimed() {
	var now = (new Date()).getTime();
	
	// last move, go to aimed position
	if(now >= this.endtime) {
		this.setY(this.aim);
	}
	
	// new position
	else {
		var delta = (now - this.starttime) / this.speed;
		var fraction = ((delta * delta) * 3.0) - ((delta * delta * delta) * 2.0);
		var curr = Math.round(this.start + (fraction * (this.aim - this.start)));
		
		this.setY(curr);
		
		clearTimeout(this.timeout);
		if(theBall.run)
			this.timeout = setTimeout(('left' == this.side) ? "leftBar.moveTimed()" : "rightBar.moveTimed()", 60);
	}
}

function playerMoveCompetition() {
	if(('game' != mode) || ('left' != this.side) || !theBall.moving)
		return;
	
	var now = (new Date()).getTime();
	var ourCenter = 1* this.y + ((this.height - theBall.size) / 2);
	var newY = this.y;
	
	var gap = now - this.starttime;
	gap = (gap > (this.competitionSpeed / 4)) ? (this.competitionSpeed / 4) : gap;
	var moveBy = Math.round((ballControl.fieldTotalHeight - this.height) / this.competitionSpeed * gap);
	
	// ball above the player
	if(theBall.y < this.y)
		newY = this.y - moveBy;
	
	// ball below player
	else if(theBall.bottomY > this.bottomY)
		newY = 1* this.y + moveBy;
	
	this.setY(newY);
	
	clearTimeout(this.timeout);
	if((!theBall.inGoal) && (-1 == theBall.direction)) {
		this.starttime = now;
		this.timeout = setTimeout("leftBar.moveCompetition()", 40);
	}
}

function playerMoveManual() {
	if(paused)
		togglePause();
	
	var now = (new Date()).getTime();
	if(keyPressed) {
		var gap = now - this.starttime;
		gap = (gap > (this.competitionSpeed / 4)) ? (this.competitionSpeed / 4) : gap;
		var moveBy = Math.round((ballControl.fieldTotalHeight - this.height) / this.competitionSpeed * gap);
		var current = this.y;
		var add = (keyPressed == "upR") ? -1 * moveBy : moveBy;
		
		// assign new top
		this.setY(1* current + add);
		
		this.starttime = now;
		clearTimeout(this.timeout);
		this.timeout = setTimeout("rightBar.moveManual()", 40);
	}
}

function playerSetCompetitionSpeed() {
	var ms = theBall.speed
	if('right' == this.side) {
		ms = playerBar.competitionSpeed * theBall.speed;
		ms = (ms > 720) ? 720 : ms;
	}
	ms = Math.round(ms);
	
	this.competitionSpeed = ms;
}







/*
 ****************************
 *      The ball-class      *
 ****************************
 */
function Ball() {
	if( ! (this instanceof Ball))		// ensure we successfully created a ball
		return new Ball();
	
	this.size = ballControl.ballSize;
	this.run = true;
	this.x = 10;
	this.y = -10;
	this.rightX = 1* this.x + this.size;
	this.bottomY = 1* this.y + this.size;
	
	this.angle = ballControl.startAngle;
	this.direction = 1;
	this.targetBar = null;
	this.moving = false;
	this.speed = ballControl.speed;
	this.projectedY = -10;
	this.lastStart = (new Date()).getTime();
	this.timeout = null;
	
	this.inGoal = false;
	
	this.div = document.createElement('div');
	this.div.setAttribute("class", "ball");
	this.div.setAttribute("style", "left:" + this.x + "px; top:" + this.y + "px;");
	getObj('playField').appendChild(this.div);
	
	this.init();
}

Ball.prototype.getX = getX;
Ball.prototype.getY = getY;
Ball.prototype.setX = ballSetX;
Ball.prototype.setY = ballSetY;
Ball.prototype.init = ballInit;
Ball.prototype.start = ballStart;
Ball.prototype.startTimed = ballStartTimed;
Ball.prototype.move = ballMove;
Ball.prototype.hitsPlayer = ballHitsPlayer;
Ball.prototype.setProjectedY = ballSetProjectedY;
Ball.prototype.outOfBounds = ballOutOfBounds;


// functions used by the Ball-Objects

function ballSetX(x) {
	if(isNaN(x))
		x = 10;
	
	this.x = x;
	this.rightX = 1* x + this.size;
	this.div.style.left = x + 'px';
}

function ballSetY(y) {
	if(isNaN(y))
		y = 10;
	
	// bounce the ball off the top and off the bottom
	while((y < 0) || (y > ballControl.fieldHeight)) {
		
		// bounce off top
		if(y < 0) {
			y = y * -1;
			this.angle *= -1;
		}
		
		// bounce off the bottom
		else if(y > ballControl.fieldHeight) {
			y = ballControl.fieldHeight - (y - ballControl.fieldHeight);
			this.angle *= -1;
		}
	}
	
	// set
	this.y = y;
	this.bottomY = 1* y + this.size;
	this.div.style.top = y + 'px';
}


// sets a random start-position for the ball
function ballInit() {
	var x = (1 == this.direction) ? (-1 * this.size) : (1* ballControl.fieldTotalWidth + this.size);
	var y = Math.round(Math.random() * ballControl.fieldHeight);		// a more or less random y-start-position
	
	this.setX(x);
	this.setY(y);
	
	this.moving = false;
	this.angle = ballControl.startAngle;
	this.inGoal = false;
}


function ballStart() {
	if(!this.run)
		return;
	
	this.lastStart = (new Date()).getTime();
	this.moving = true;
	this.move();
	
	if(-1 == this.direction) {
		clearTimeout(leftBar.timeout);
		leftBar.timeout = setTimeout("leftBar.startAutoMove()", Math.round(this.speed / 5));
	}
}


function ballStartTimed() {
	if("clock" != mode)
		return;
	
	var second = (new Date()).getSeconds();
	
	// while we count the seconds, place the ball and call start, which moves the ball
	if((second > 3) && (second < 58)) {
		this.init();
		this.setProjectedY();
		this.start();
		
		if(1 == this.direction) {
			clearTimeout(rightBar.timeout);
			setTimeout("rightBar.startAutoMove()", Math.round(this.speed / 5));
		}
		else {
			clearTimeout(leftBar.timeout);
			setTimeout("leftBar.startAutoMove()", Math.round(this.speed / 5));
		}
	}
		
	// one minute passed
	else {
		var milli = (new Date()).getMilliseconds();
		
		setTimeout("showTime()", 1000);
		setTimeout("theBall.startTimed()", (1000 * (4 - (second % 58))) - milli);		// re-calls itself to relaunch on the 4th second of the minute
	}
}


function ballMove() {
	if(!this.run || !this.moving)
		return;
	
	var now = (new Date()).getTime();
	milli = now - this.lastStart;					// how long did it take from the last move?
	milli = (milli == 0) ? 1 : milli;					// remove a 0-ms-offset
	milli = (milli > (this.speed / 3)) ? 50 : milli;					// if it took too long from last call (paused, maybe?), simply screw =D
	var fraction = Math.abs(milli / this.speed);			// get the fraction (the amount we need to move of the total playfield-width)
	
	// calc a new x-value
	var deltaX = Math.round(ballControl.fieldWidth * fraction);
	var nextX = (1* this.x) + (this.direction * deltaX);
	
	// calculate the Y-value
	var deltaY = getDeltaY(deltaX, true);
	var nextY = 1* this.y + deltaY;
	
	this.setY(nextY);
	this.setX(nextX);
	
	// handle bounces from the Players
	this.hitsPlayer();
	
	// handle ball out of bounds
	this.outOfBounds();
	
	clearTimeout(this.timeout);
	if(this.run && this.moving) {
		this.timeout = setTimeout("theBall.move()", 40);
		this.lastStart = now;
	}
}


function ballHitsPlayer() {
	this.targetBar = (this.direction == 1) ? rightBar : leftBar;
	var barX = (1 == this.direction) ? this.targetBar.x : this.targetBar.rightX;
	var myX = (1 == this.direction) ? this.rightX : this.x;
	
	// check for x
	if((1 == this.direction) ? (myX >= barX) : (myX <= barX)) {
		
		// is this the first time the ball is behind the bar?
		if(!this.inGoal) {
			var myY = this.y;
			var deltaX = Math.abs(myX - barX);
			if(deltaX > (0.75 * ballControl.bounceDist)) {
				this.inGoal = true;
				return false;
			}
			
			// correct the y-value to the level of the bar (since the ball moves more than one x-pixel per step)
			var deltaY = 0;
			if(deltaX > 1) {
				deltaY = getDeltaY(deltaX);
				myY -= deltaY;
			}
			
			// check for y
			if((this.targetBar.y <= (this.bottomY - deltaY)) && (this.targetBar.bottomY >= myY)) {
				this.setX(this.targetBar.surfaceX);
				var hitHeight = 1* this.targetBar.height + this.size;		// the full height the ball can hit
				var absY = myY - (this.targetBar.y - this.size);		// how far away (from center view) the ball hits
				var part = 1;										// 1 = hit in upper part, -1 = hit in lower part of the bar
				if(absY > (hitHeight / 2)) {
					absY = hitHeight - absY;
					part = -1;
				}
				
				// change angle absolute to the hit-position
				if(('absolute' == bouncemode) && (absY < (this.targetBar.height / 2))) {
					var centerOfBar = hitHeight / 2;
					var deltaHit = (1 - (absY / centerOfBar));
					this.angle = Math.round(deltaHit * ballControl.maxAngle * part * -1);
				}
				
				// change angle relative to the hit-position
				else if(absY < (this.targetBar.height / 2)) {
					var deltaHit = (1 - (absY / (this.targetBar.height / 2)));
					
					var deltaAngle = Math.round(this.targetBar.anglePercentageChange * (90 - (part * this.angle)));
					deltaAngle = Math.round(deltaHit * deltaAngle * part * -1);
					
					this.angle = this.angle + (1* deltaAngle);
				}
				
				// check for max angle
				if(this.angle < (-1 * ballControl.maxAngle))
					this.angle = (-1 * ballControl.maxAngle);
				else if(this.angle > ballControl.maxAngle)
					this.angle = ballControl.maxAngle;
				
				// bounce and set necessary vars
				this.direction *= -1;
				this.setProjectedY();
				
				if(1 == this.direction) {
					clearTimeout(rightBar.timeout);
					setTimeout("rightBar.startAutoMove()", Math.round(this.speed / 5));
				}
				else {
					clearTimeout(leftBar.timeout);
					setTimeout("leftBar.startAutoMove()", Math.round(this.speed / 5));
				}
				
				return true;
			}
			
			else {
				this.inGoal = true;
			}
		}
		
		// bounce off top or bottom of the bar (eyecandy =D )
		else if((1 == this.direction) ? (this.x < this.targetBar.rightX) : (this.rightX > this.targetBar.x)) {
			if((this.angle < 0) ?
								((this.y > (this.targetBar.bottomY - 6)) && (this.y < (1 * this.targetBar.bottomY + 6))) :
								((this.bottomY > (this.targetBar.y - 6)) && (this.bottomY < (1 * this.targetBar.y + 6)))) {
				this.angle *= -1;
			}
		}
	}
	return false;
}


function ballSetProjectedY() {
	var deltaX = (this.direction == 1) ?
						(1* ballControl.fieldWidth + ballControl.bounceDist - this.x) : 
						(this.x - ballControl.bounceDist);		// distance from current ball-position to surface of the matching playerbar
	var projY = (this.y * 1) + getDeltaY(deltaX);
	while((projY < 0) || (projY > ballControl.fieldHeight)) {
		
		// top
		if(projY < 0)
			projY = projY * -1;
		
		// bottom
		else if(projY > ballControl.fieldHeight)
			projY = Math.abs(ballControl.fieldHeight - (projY - ballControl.fieldHeight));
	}
	
	this.projectedY = projY;
}


// checks if the ball is still on the playing-field (checks only x, since the ball bounces from the top and bottom). If the ball`s out, sets it to the opposite side.
function ballOutOfBounds() {
	if((-1 == this.direction && this.x < (this.size * -1)) || (1 == this.direction && this.x > ballControl.fieldTotalWidth)) {
		this.setX((1 == this.direction) ? (ballControl.ballSize * -1) : (1* ballControl.fieldTotalWidth + ballControl.ballSize));
		this.moving = false;
		
		if("clock" == mode) {
			this.startTimed();
		}
		else {
			updateScore();
		}
		
		return true;
	}
		
	return false;
}





/*
 **********************
 *      by Apple      *
 **********************
 */

function mousemove(event) {
	if(! flipShown) {
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		var starttime = (new Date).getTime() - 13;
 
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = getObj("flip");
		animation.secondElement = getObj("modeToggle");
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}


function mouseexit (event) {
	if (flipShown) {
		// fade in the info button
		if (animation.timer != null) {
			clearInterval (animation.timer);
			animation.timer  = null;
		}

		var starttime = (new Date).getTime() - 13;

		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = getObj("flip");
		animation.secondElement = getObj("modeToggle");
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate() {
	var T;
	var ease;
	var time = (new Date).getTime();
   

	T = limit_3(time-animation.starttime, 0, animation.duration);

	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}

	animation.firstElement.style.opacity = animation.now;
	animation.secondElement.style.opacity = animation.now;
}

function limit_3 (a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
	return from + (to - from) * ease;
}

// these functions are called when the info button itself receives onmouseover and onmouseout events (by Apple)
// I modified them to inverse the colors on hover

function enterflip(event) {
	getObj("flip").style.backgroundImage = "url(/System/Library/WidgetResources/ibutton/white_rollie.png)";
	getObj("flippi").src = "/System/Library/WidgetResources/ibutton/black_i.png";
}

function exitflip(event) {
	getObj("flip").style.backgroundImage = "url(/System/Library/WidgetResources/ibutton/black_rollie.png)";
	getObj("flippi").src = "/System/Library/WidgetResources/ibutton/white_i.png";
}

